@@ -2,15 +2,18 @@
 //!
 //! <https://en.wikipedia.org/wiki/Particle_swarm_optimization>
 use crate::utility::prelude::*;
+use alloc::vec::Vec;
+use core::marker::PhantomData;
 
 /// Particle Swarm Optimization settings.
-pub struct Pso {
+pub struct Pso<F: Fitness> {
     cognition: f64,
     social: f64,
     velocity: f64,
+    _marker: PhantomData<F>,
 }
 
-impl Pso {
+impl<F: Fitness> Pso<F> {
     impl_builders! {
         /// Cognition factor.
         fn cognition(f64)
@@ -21,18 +24,19 @@ impl Pso {
     }
 }
 
-impl Default for Pso {
+impl<F: Fitness> Default for Pso<F> {
     fn default() -> Self {
         Self {
             cognition: 2.05,
             social: 2.05,
             velocity: 1.3,
+            _marker: PhantomData,
         }
     }
 }
 
-impl Setting for Pso {
-    type Algorithm = Method;
+impl<F: Fitness> Setting for Pso<F> {
+    type Algorithm = Method<F>;
 
     fn algorithm(self) -> Self::Algorithm {
         Method {
@@ -40,32 +44,29 @@ impl Setting for Pso {
             social: self.social,
             velocity: self.velocity,
             best_past: Array2::zeros((1, 1)),
-            best_past_f: Array1::ones(1) * f64::INFINITY,
+            best_past_f: Vec::new(),
         }
     }
 
-    fn default_basic() -> BasicSetting {
-        BasicSetting {
-            pop_num: 200,
-            ..Default::default()
-        }
+    fn default_pop() -> usize {
+        200
     }
 }
 
 /// Particle Swarm Optimization type.
-pub struct Method {
+pub struct Method<F: Fitness> {
     cognition: f64,
     social: f64,
     velocity: f64,
     best_past: Array2<f64>,
-    best_past_f: Array1<f64>,
+    best_past_f: Vec<F>,
 }
 
-impl<F: ObjFunc> Algorithm<F> for Method {
+impl<F: ObjFunc> Algorithm<F> for Method<F::Fitness> {
     #[inline(always)]
     fn init(&mut self, ctx: &mut Context<F>) {
         self.best_past = ctx.pool.clone();
-        self.best_past_f = Array1::from_iter(ctx.fitness.iter().map(|r| r.value()));
+        self.best_past_f = ctx.fitness.clone();
     }
 
     fn generation(&mut self, ctx: &mut Context<F>) {
@@ -88,9 +89,9 @@ impl<F: ObjFunc> Algorithm<F> for Method {
                         + beta * (ctx.best[s] - v[s]);
                     v[s] = ctx.check(s, variable);
                 }
-                *f = ctx.func.fitness(v.as_slice().unwrap(), &ctx.report);
-                if f.value() < *f_past {
-                    *f_past = f.value();
+                *f = ctx.func.fitness(v.as_slice().unwrap(), ctx.adaptive);
+                if *f < *f_past {
+                    *f_past = f.clone();
                     past.assign(&v);
                 }
             });
@@ -109,16 +110,16 @@ impl<F: ObjFunc> Algorithm<F> for Method {
                             + beta * (ctx.best[s] - v[s]);
                         v[s] = ctx.check(s, variable);
                     }
-                    *f = ctx.func.fitness(v.as_slice().unwrap(), &ctx.report);
-                    if f.value() < *past_f {
-                        *past_f = f.value();
+                    *f = ctx.func.fitness(v.as_slice().unwrap(), ctx.adaptive);
+                    if *f < *past_f {
+                        *past_f = f.clone();
                         past.assign(&v);
                     }
-                    (f.value(), v)
+                    (f, v)
                 })
                 .min_by(|(a, _), (b, _)| a.partial_cmp(b).unwrap())
                 .unwrap();
-            ctx.set_best_from(f, &v);
+            ctx.set_best_from(f.clone(), &v);
         }
         ctx.fitness = fitness;
         ctx.pool = pool;
